/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.medor.clone.api;

/**
 * Defines a Cloneable object. The simple way to launch the cloning of an object
 * graph is to call 'myObj.clone(null, new HashMap())'.
 *
 * @author S.Chassande-Barrioz
 */
public interface Cloneable extends java.lang.Cloneable {

	/**
	 * Clone the current object.
	 * @param clone is the clone of the current object or the null value. The
	 * interest of this parameter is to permit to an implementation to specify
	 * its instance and in particular to chose how the instance is created
	 * (which constructor). If this parameter is not null, it must be an
	 * instance of the current object.
	 * @param obj2clone is the context of the cloning. It contains the
	 * association between an object and its clone. When an object is cloned,
	 * it should be registered with its clone into the map. By this way object
	 * graph could be cloned without having two different clones for the same
	 * source object
	 * @return a clone instance of the current object. If the clone parameter is
	 * not null, the returned object is the parameter. Otherwise the
	 * implementation of the method must instanciate a clone in order to never
	 * retrieve a null value.
	 *
	 * @throws CloneNotSupportedException if an error occurs during the cloning
	 * step.
	 */
	Object clone(Object clone, java.util.Map obj2clone) throws CloneNotSupportedException;
}
