/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.expression.lib;

import org.objectweb.medor.expression.api.BinaryOperator;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.MalformedExpressionException;
import org.objectweb.medor.expression.api.Operand;
import org.objectweb.medor.expression.api.Operator;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.TypingException;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;

public class FirstLocate extends BasicBinaryOperator implements BinaryOperator {

    private static final long serialVersionUID = 240849460005274957L;

    /**
	 * Returns the first position of an Expression (first parameter) in another
	 * Expression (second parameter).
	 * <p>Unlike in Java, the index is 1 for the first position, and not zero.
	 * @param substring is the Expression to be tested as a subexpression
	 * @param instring is the Expression which may contain the expressions[0] Expression.
	 */
	public FirstLocate(Expression substring, Expression instring) {
		super(PTypeSpace.INT, substring, instring);
	}

	public FirstLocate() {
        super(PTypeSpace.INT);
	}

	public Operand evaluate(ParameterOperand[] pos, Object o)
		throws ExpressionException {
        if (result == null) {
            throw new IllegalStateException("Unevaluable Expression: Not compiled");
        }
        result.setValue(evaluate(
                expressions[0].evaluate(pos, o).getString(),
                expressions[1].evaluate(pos, o).getString()));
		return result;
	}

	public int evaluate(String op1, String op2) {
		return op2.indexOf(op1) + 1;
	}

	public Operand compileExpression()
		throws ExpressionException, MalformedExpressionException {
        compileExpression();
        checkOperands(new PType[][]{
                {PTypeSpace.STRING, PTypeSpace.STRING},
        });
        verified = true;
        result = new BasicVariableOperand(type);
        return result;
	}

	public String getOperatorString() {
		return Operator.FIRSTLOCATE;
	}
}
