/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.expression.lib;

import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.MalformedExpressionException;
import org.objectweb.medor.expression.api.Operand;
import org.objectweb.medor.expression.api.Operator;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.TypingException;
import org.objectweb.medor.expression.api.UnaryOperator;

public class Length extends BasicUnaryOperator implements UnaryOperator {

    private static final long serialVersionUID = 102154146175607290L;

    public Length() {
    }

    public Length(Expression e) {
        super(PTypeSpace.INT, e);
    }

    public Operand evaluate(ParameterOperand[] pos, Object o)
            throws ExpressionException {
        if (result == null) {
            throw new ExpressionException("Unevaluable Expression: Not compiled");
        }
        result.setValue(evaluate(expressions[0].evaluate(pos, o).getString()));
		return result;
    }

    public int evaluate(String op) {
        return op.length();
    }

    public Operand compileExpression()
            throws ExpressionException, MalformedExpressionException {
        compileOperands();
        checkOperands(new PType[][]{ {PTypeSpace.STRING} });
        result = new BasicVariableOperand(type);
        verified = true;
 		return result;
    }

    public String getOperatorString() {
        return Operator.LENGTH;
    }
}
